/* 
    L64781  -  DVB-T COFDM demodulator used in Technotrend DVB-T cards

    Copyright (C) 1999 Convergence Integrated Media GmbH

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/    

#include <linux/module.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/poll.h>
#include <linux/i2c.h>

#include "dvb.h"
#include "dvb_demodulator.h"


static struct i2c_driver dvbt_driver;
static struct i2c_client client_template;

struct L64781 {
	__u32 srate;  /*  dummy for now ... */
};



static __u8 L64781_init_regdump [] = {
	0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x02, 0x86,
	0x00, 0x00, 0x00, 0x80, 0x8c, 0x0e, 0x00, 0x6c,
	0x00, 0x40, 0x00, 0x00, 0x44, 0x03, 0x52, 0x27,
	0x00, 0x92, 0xa1, 0xe0, 0x96, 0x5b, 0x49, 0x12,
	0x36, 0x36, 0xfe, 0x3f, 0x25, 0x2c, 0xd1, 0xfc,
	0xd1, 0x00, 0x81, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa5
};



static
int write_reg(struct i2c_client *client, __u8 reg, __u8 data)
{
	int ret;
	__u8 msg[2] = { reg, data };

	if (reg < 0x02 || (reg > 0x31 && reg != 0x3e)) {
		printk ("L64781: try to write readonly register 0x%02x !", reg);
		return -1;
	}
	
	if ((ret = i2c_master_send (client, msg, 2)) != 2)
		printk ("L64781: write_reg error !\n");

	mdelay(10);

	return ret;
}


static
__u8 read_reg(struct i2c_client *client, __u8 reg)
{
        struct i2c_adapter *adap=client->adapter;
        unsigned char mm1[] = {reg};
        unsigned char mm2[] = {0x00};
        struct i2c_msg msgs[2];

        msgs[0].flags=0;
        msgs[1].flags=I2C_M_RD;
        msgs[0].addr=msgs[1].addr=client->addr;
        msgs[0].len=1; msgs[1].len=1;
        msgs[0].buf=mm1; msgs[1].buf=mm2;
        i2c_transfer(adap, msgs, 2);

        return mm2[0];
}


static
void L64781_dump_regs (struct i2c_client *client)
{
	int i;

	printk ("%s:\n", __FUNCTION__);
	printk ("static __u8 L64781_regdump [] = {");
	
	for (i=0; i<=0x3e; i++) {
		if ((i & 0x7) == 0)
			printk ("\n\t");

		printk ("0x%02x", read_reg (client, i));

		if (i < 0x3e)
			printk (", ");
	}

	printk ("\n};\n");
}




static
int init(struct i2c_client *client)
{
        struct i2c_adapter *adap=client->adapter;

        (void) adap;
        printk("L64781: init chip\n");

	L64781_dump_regs (client);
#if 0
  	for (i=0x2; i<=0x31; i++)
  		write_reg (client, i, L64781_init_regdump[i]);
#endif
	write_reg (client, 0x3e, L64781_init_regdump[0x3e]);
	
	L64781_dump_regs (client);

	return 0;
}



static
int attach_adapter(struct i2c_adapter *adap)
{
        struct i2c_client *client;
	__u8 tmp_addr = client_template.addr;
	__u8 msg = 0x06;

	client_template.adapter=adap;

	if (i2c_master_send(&client_template,NULL,0) == 0)
                return -1;      /*  no L64781, proabably an EEPROM ... */

	client_template.addr = 0;
	i2c_master_send(&client_template, &msg, 1);
	client_template.addr = tmp_addr;

	if (i2c_master_send(&client_template,NULL,0))
                return -1;      /*  no L64781 found ... */

        client_template.adapter=adap;
        
        if (!(client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL)))
                return -ENOMEM;
        
        memcpy(client, &client_template, sizeof(struct i2c_client));

	if (!(client->data = kmalloc(sizeof(struct L64781), GFP_KERNEL))) {
                kfree (client);
                return -ENOMEM;
        }
       
        printk("L64781: attaching L64781 at 0x%02x\n", (client->addr)<<1);

        i2c_attach_client(client);
        
        init(client);
        
	MOD_INC_USE_COUNT;
        
        return 0;
}


static
int detach_client(struct i2c_client *client)
{
        printk("L64781: detach_client\n");
        i2c_detach_client(client);
        kfree(client->data);
        kfree(client);
	MOD_DEC_USE_COUNT;
        return 0;
}

static 
int dvb_command(struct i2c_client *client, unsigned int cmd, void *arg)
{
        switch (cmd) {
        case DVB_WRITEREG:
	{
		__u8 *msg = (__u8 *) arg;
		printk ("%s: DVB_WRITEREG %#x := %#x\n",
			__FUNCTION__, msg[0], msg[1]);
//		write_reg(client, msg[0], msg[1]);
		break;
	}
	case DVB_READREG:
        {
                __u8 *msg = (__u8 *) arg;
                msg[1] = read_reg (client, msg[0]);
                break;
        }
	case DVB_INIT_FRONTEND:
	{
		printk ("%s: DVB_INIT_FRONTEND\n", __FUNCTION__);
                break;
	}
        case DVB_RESET:
        {
		printk ("%s: DVB_RESET\n", __FUNCTION__);
                break;
	}
	case DVB_SET_FRONTEND:
        {
                struct frontend *front = (struct frontend *)arg;
                
		printk ("%s: DVB_SET_FRONTEND\n", __FUNCTION__);

		if (front->flags & FRONT_FREQ_CHANGED)
                        ;
		break;
        }
        case DVB_GET_FRONTEND:
        {
                struct frontend *front = (struct frontend *)arg;
                
		printk ("%s: DVB_GET_FRONTEND\n", __FUNCTION__);

		front->afc = read_reg(client, 0x08)
		          | (read_reg(client, 0x09) << 8)
		          | (read_reg(client, 0x0a) << 16);
		front->agc = read_reg(client,0x0c) & 0x7;
		front->fsync = (read_reg(client,0x32) >> 3 ) & 0x1;
		front->sync = read_reg(client,0x32) & 0x1;
		front->nest = 0;
		front->vber = read_reg(client, 0x39)
			   | (read_reg(client, 0x3a) << 8);
		printk ("L64781: afc == %i\n", front->afc);
		printk ("L64781: agc == %i\n", front->agc);
		printk ("L64781: agc == %#x\n", read_reg(client,0x0c));
		printk ("L64781: fsync == %i\n", front->fsync);
		printk ("L64781: sync == %i\n", front->sync);
		printk ("L64781: nest == %i\n", front->nest);
		printk ("L64781: vber == %i\n", front->vber);
		break;
        }
        default:
		printk ("%s: unknown command !!!\n", __FUNCTION__);
                return -1;
        };
        
        return 0;
} 

void inc_use (struct i2c_client *client)
{
#ifdef MODULE
        MOD_INC_USE_COUNT;
#endif
}

void dec_use (struct i2c_client *client)
{
#ifdef MODULE
        MOD_DEC_USE_COUNT;
#endif
}

static
struct i2c_driver dvbt_driver = {
	name: 		"L64781 DVB demodulator",
	id:		I2C_DRIVERID_L64781,
	flags:		I2C_DF_NOTIFY,
	attach_adapter:	attach_adapter,
	detach_client:	detach_client,
	command:	dvb_command,
	inc_use:	inc_use,
	dec_use:	dec_use
};

static
struct i2c_client client_template = {
        name:    "L64781",
        id:      I2C_DRIVERID_L64781,
        flags:   0,
        addr:    0x55,
        adapter: NULL,
        driver:  &dvbt_driver
};


static __init
int init_l64781 (void) {
        int res;
        
        if ((res = i2c_add_driver(&dvbt_driver))) 
        {
                printk("L64781: i2c driver registration failed\n");
                return res;
        }

        printk("L64781: init done\n");
        return 0;
}

static __exit
void exit_l64781 (void)
{
        int res;

        if ((res = i2c_del_driver(&dvbt_driver))) 
        {
                printk("L64781: Driver deregistration failed, "
                       "module not removed.\n");
        }
        printk("L64781: cleanup\n");
}

module_init(init_l64781);
module_exit(exit_l64781);
